VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "TextParser"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private Type FieldData
  Field As String
  Value As Variant
End Type
Private Fields() As FieldData, vFieldCount As Long

Private Const NullValue As String = vbNullChar & vbNullChar, vbSpace As String = " "

Private Const TagEscape As String = "\"
Private Const TagQuote As String = "'"
Private Const TagField As String = "%"
Private Const TagSeparator As String = ";"
Private Const TagOpen As String = "[", TagClose As String = "]"
Private Const TagFunction As String = "$"
Private Const TagFSeparator As String = ","
Private Const TagFOpen As String = "(", TagFClose As String = ")"

Private Const WordCharacters As String = "ABCDEFGHIJKLMNOPQRSTUVWXYZ_-0123456789"
Private Const WordSeparator As String = " ,;:-.!?"

Private Const NameSpacer As String = " "
Private Const NameHyphen As String = "-"

Private vFormatString As String, vFieldIndex As String
Private fBusy As Boolean

Private Sub Class_Initialize()
ReDim Fields(0)
vFieldCount = 0
vFormatString = vbNullString
vFieldIndex = vbNullString
End Sub

Private Sub Class_Terminate()
Erase Fields()
vFormatString = vbNullString
vFieldIndex = vbNullString
End Sub

Public Sub FieldClear()
ReDim Fields(0)
vFieldCount = 0
vFieldIndex = vbNullString
End Sub

Public Property Get FieldCount() As Long
FieldCount = vFieldCount = 0
End Property

Public Sub FieldAdd(ByVal FieldName As String, Optional ByVal FieldValue As Variant = Null, Optional ByVal AutoUpdate As Boolean = False)
Dim I As Long, S As String
If vFieldCount >= 255 Then Exit Sub
S = vbTab & UCase$(FieldName) & vbNullChar
I = InStr(1, vFieldIndex, S, vbBinaryCompare)
If I > 0 Then I = Asc(Mid$(vFieldIndex, I - 1, 1))
If I = 0 Then
  I = vFieldCount + 1
  ReDim Preserve Fields(0 To I)
  vFieldCount = I
  vFieldIndex = vFieldIndex & Chr$(I) & vbTab & UCase$(FieldName) & vbNullChar
  Fields(I).Field = FieldName
Else
  If Not AutoUpdate Then Exit Sub
End If
Fields(I).Value = FieldValue
End Sub

Public Sub FieldDelete(ByVal FieldName As String)
Dim N As Long, I As Long, S As String
If vFieldCount = 0 Then Exit Sub
S = vbTab & UCase$(FieldName) & vbNullChar
I = InStr(1, vFieldIndex, S, vbBinaryCompare) - 1
If I > 0 Then N = Asc(Mid$(vFieldIndex, I, 1))
If I <= 0 Then Exit Sub
If N = vFieldCount Then
  vFieldIndex = Left$(vFieldIndex, I - 1)
Else
  vFieldIndex = vbNullString
  For I = 1 To vFieldCount
    Select Case I
      Case Is < N
        vFieldIndex = vFieldIndex & Chr$(I) & vbTab & UCase$(Fields(I).Field) & vbNullChar
      Case Is = N
      Case Is > N
        vFieldIndex = vFieldIndex & Chr$(I - 1) & vbTab & UCase$(Fields(I).Field) & vbNullChar
    End Select
  Next I
End If
For I = N To vFieldCount - 1
  Fields(I) = Fields(I + 1)
Next I
ReDim Preserve Fields(0 To vFieldCount - 1)
vFieldCount = vFieldCount - 1
End Sub

Public Function FieldExist(ByVal FieldName As String) As Boolean
FieldExist = (InStr(1, vFieldIndex, vbTab & UCase$(FieldName) & vbNullChar, vbBinaryCompare) > 0)
End Function

Public Property Get Field(ByVal FieldName As String) As Variant
Dim I As Long, S As String
If vFieldCount = 0 Then Exit Property
S = vbTab & UCase$(FieldName) & vbNullChar
I = InStr(1, vFieldIndex, S, vbBinaryCompare)
If I > 0 Then I = Asc(Mid$(vFieldIndex, I - 1, 1))
If I > 0 Then Field = Fields(I).Value
End Property

Public Property Let Field(ByVal FieldName As String, ByVal FieldValue As Variant)
Dim I As Long, S As String
If vFieldCount = 0 Then Exit Property
S = vbTab & UCase$(FieldName) & vbNullChar
I = InStr(1, vFieldIndex, S, vbBinaryCompare)
If I > 0 Then I = Asc(Mid$(vFieldIndex, I - 1, 1))
If I > 0 Then Fields(I).Value = FieldValue
End Property

Public Sub FieldLoadText(ByVal DataSource As String, Optional ByVal PreClear As Boolean = True)
Dim I0 As Long, I As Long, P As Long
Const FS As String = vbTab, rs As String = vbNullChar
If PreClear Then FieldClear
I0 = 1
Do Until I0 > Len(DataSource)
  I = InStr(I0, DataSource, rs)
  If I = 0 Then Exit Do
  P = InStr(I0, DataSource, FS)
  If P > 0 Then
    FieldAdd Mid$(DataSource, I0, P - I0), (Mid$(DataSource, P + Len(FS), I - P - Len(FS))), True
  End If
  I0 = I + Len(rs)
Loop
End Sub

Public Function FieldSaveText() As String
Dim I As Long, res As String
Const FS As String = vbTab, rs As String = vbNullChar
For I = 1 To vFieldCount
  res = res & Fields(I).Field & FS & CStr(Fields(I).Value) & rs
Next I
FieldSaveText = res
End Function

Public Sub FieldLoadADO(Recordset As Object, Optional ByVal PreClear As Boolean = True)
Dim I As Long
If PreClear Then FieldClear
For I = 0 To Recordset.Fields.Count - 1
  FieldAdd Recordset.Fields(I).Name, Recordset.Fields(I).Value, True
Next I
End Sub

Public Property Get FormatString() As String
FormatString = vFormatString
End Property

Public Property Let FormatString(ByVal NewFormat As String)
vFormatString = NewFormat
End Property

Private Function GetField(ByVal FieldName As String) As String
Dim I As Long, S As String
GetField = NullValue
If vFieldCount = 0 Then Exit Function
S = vbTab & UCase$(FieldName) & vbNullChar
I = InStr(1, vFieldIndex, S, vbBinaryCompare)
If I > 0 Then I = Asc(Mid$(vFieldIndex, I - 1, 1))
If I > 0 Then
  If Not IsNull(Fields(I).Value) Then
    GetField = Fields(I).Value
  End If
End If
End Function

Public Function SubstText(Optional ByVal sFormat As String, Optional ByVal SkipIsNullable As Boolean = False) As String
Dim res As String, PosStart As Long, FieldsTotal As Long, FieldsNull As Long, F As String, Args() As Variant
Dim P As Long, C As String, N As Long, tmpIndex As Long, tmpString As String, fProcessed As Boolean
If Len(sFormat) = 0 Then sFormat = vFormatString
PosStart = 1
res = vbNullString
FieldsTotal = 0&
FieldsNull = 0&
Do Until PosStart > Len(sFormat)
  Call FirstInStr(P, C, PosStart, sFormat, TagEscape, TagQuote, TagField, TagFunction, TagOpen)
  If P = 0 Then
    res = res & Mid$(sFormat, PosStart)
    Exit Do
  End If
  res = res & Mid$(sFormat, PosStart, P - PosStart)
  PosStart = P + Len(C)
  Select Case C
    Case TagEscape
      Select Case UCase$(Mid$(sFormat, PosStart, 1))
        Case "0" ' \0 - null = \*0x00*
          res = res & vbNullChar
        Case "T" ' \t - <TAB>, 0x09 = \*0x09*
          res = res & vbTab
        Case "N" ' \n - <CR><LF>, 0x0D0A = \*0x0D0A*
          res = res & vbNewLine
        Case "*" ' \*<code>* - ascii char with code; code is decimal (#) or hex (0x##)
          P = InStr(PosStart, sFormat, "*", vbBinaryCompare)
          If P = 0 Then P = Len(sFormat) + 1
          tmpString = UCase$(Mid$(sFormat, PosStart + 1, P - (PosStart + 1)))
          PosStart = P - 1
          If Left$(tmpString, 2) = "0X" Then
            tmpString = Mid$(tmpString, 3)
            If (Len(tmpString) And 1&) = 1& Then tmpString = "0" & tmpString
            For tmpIndex = 1 To Len(tmpString) Step 2
              res = res & Chr$(Val("&h" & Mid$(tmpString, tmpIndex, 2)))
            Next tmpIndex
          Else
            res = res & Chr$(Val(tmpString))
          End If
        Case Else
          res = res & Mid$(sFormat, PosStart, 1)
      End Select
      PosStart = PosStart + 1
    Case TagQuote
      P = PosStart
      Do
        P = InStr(P, sFormat, C, vbBinaryCompare)
        If P = 0 Then
          P = Len(sFormat) + 1
          Exit Do
        End If
        If Mid$(sFormat, P - Len(TagEscape), Len(TagEscape)) <> TagEscape Then Exit Do
        P = P + Len(C)
      Loop
      tmpString = Mid$(sFormat, PosStart, P - PosStart)
      PosStart = P + Len(C)
      P = 1
      Do
        tmpIndex = InStr(P, tmpString, TagEscape, vbBinaryCompare)
        If tmpIndex = 0 Then
          res = res & Mid$(tmpString, P)
          Exit Do
        End If
        P = tmpIndex + Len(TagEscape)
        Select Case UCase$(Mid$(tmpString, P, 1))
          Case "0" ' \0 - null
            res = res & vbNullChar
          Case "T" ' \t - <TAB>, 0x09
            res = res & vbTab
          Case "N" ' \n - <CR><LF>, 0x0D0A
            res = res & vbNewLine
          Case Else
            res = res & Mid$(sFormat, P, 1)
        End Select
        P = P + 1
      Loop
    Case TagField
      P = InStr(PosStart, sFormat, C, vbBinaryCompare)
      If P = 0 Then P = Len(sFormat) + 1
      F = GetField(Mid$(sFormat, PosStart, P - PosStart))
      PosStart = P + Len(C)
      FieldsTotal = FieldsTotal + 1
      If F = NullValue Then FieldsNull = FieldsNull + 1 Else res = res & F
    Case TagFunction
      For P = PosStart To Len(sFormat)
        If InStr(1, WordCharacters, UCase$(Mid$(sFormat, P, 1)), vbBinaryCompare) = 0 Then Exit For
      Next P
      F = UCase$(Mid$(sFormat, PosStart, P - PosStart))
      PosStart = P
      FieldsTotal = FieldsTotal + 1
      If Mid$(sFormat, PosStart, Len(TagFOpen)) = TagFOpen Then
        P = FindPairPos(PosStart, sFormat)
        If P = 0 Then
          PosStart = PosStart + Len(TagFOpen)
        Else
          tmpString = Mid$(sFormat, PosStart + Len(TagFOpen), P - (PosStart + Len(TagFOpen)))
          Call GetArguments(Args(), tmpString)
          PosStart = P + Len(TagFClose)
          N = UBound(Args)
          For tmpIndex = 1 To N
            tmpString = Args(tmpIndex)
            fProcessed = False
            If Left$(tmpString, Len(TagField)) = TagField And Right$(tmpString, Len(TagField)) = TagField Then
              tmpString = Mid$(tmpString, 1 + Len(TagField), Len(tmpString) - Len(TagField) - Len(TagField))
              If InStr(tmpString, TagField) = 0 Then
                Args(tmpIndex) = Field(tmpString)
                fProcessed = True
              End If
            End If
            If Not fProcessed Then Args(tmpIndex) = SubstText(tmpString)
          Next tmpIndex
          Select Case N
            Case 0:     F = Func(F)
            Case 1:     F = Func(F, Args(1))
            Case 2:     F = Func(F, Args(1), Args(2))
            Case 3:     F = Func(F, Args(1), Args(2), Args(3))
            Case Else:  F = Func(F, Args(1), Args(2), Args(3), Args(4))
          End Select
          Erase Args()
        End If
      Else
        F = Func(F)
      End If
      If F = NullValue Then FieldsNull = FieldsNull + 1 Else res = res & F
    Case TagOpen
      P = FindPairPos(PosStart - Len(C), sFormat)
      If P > 0 Then
        tmpString = Mid$(sFormat, PosStart, P - PosStart)
        PosStart = P + Len(TagClose)
        res = res & SubstText(tmpString, True)
      End If
  End Select
Loop
If SkipIsNullable Then
  If FieldsTotal > 0 And FieldsNull = FieldsTotal Then res = vbNullString
End If
SubstText = res
End Function

Private Sub FirstInStr(ByRef retPos As Long, ByRef retChar As String, ByVal Start As Long, ByVal Text As String, ParamArray Chars() As Variant)
Dim I As Long, P As Long
retPos = 0&
For I = LBound(Chars) To UBound(Chars)
  P = InStr(Start, Text, Chars(I), vbBinaryCompare)
  If P > 0 And (P < retPos Or retPos = 0&) Then
    retPos = P
    retChar = Chars(I)
  End If
Next I
End Sub

Private Function FindPairPos(ByVal Start As Long, ByVal Text As String) As Long
Dim P As Long, C As String, F As String
Dim vPair0 As Boolean   ' quote (TagQuote)
Dim vPair1 As Long      ' [ ]   (TagOpen, TagClose)
Dim vPair2 As Long      ' ( )   (TagFOpen, TagFClose)
If Len(Text) < Start Then Exit Function
Select Case Mid$(Text, Start, 1)
  Case TagOpen
    F = TagClose
  Case TagFOpen
    F = TagFClose
  Case Else
    F = vbNullString
End Select
Do
  Call FirstInStr(P, C, Start, Text, TagEscape, TagQuote, TagOpen, TagClose, TagFOpen, TagFClose, TagFSeparator)
  If P = 0 Then Exit Do
  Start = P + Len(C)
  Select Case C
    Case TagEscape
      Select Case UCase$(Mid$(Text, Start, 1))
        Case "0", "T", "N"
          Start = Start + 1
        Case "*"
          If InStr(Start, Text, "*", vbBinaryCompare) = 0 Then
            Start = Start + 1
          Else
            Start = InStr(Start, Text, "*", vbBinaryCompare) + 1
          End If
        Case Else
          Start = Start + 1
      End Select
    Case TagQuote
      vPair0 = Not vPair0
    Case TagOpen
      If Not vPair0 Then
        vPair1 = vPair1 + 1
      End If
    Case TagClose
      If Not vPair0 Then
        vPair1 = vPair1 - 1
        If (F = C) And ((vPair1 + vPair2) = 0) Then
          FindPairPos = P
          Exit Function
        End If
      End If
    Case TagFOpen
      If Not vPair0 Then
        vPair2 = vPair2 + 1
      End If
    Case TagFClose
      If Not vPair0 Then
        vPair2 = vPair2 - 1
        If (F = C) And ((vPair1 + vPair2) = 0) Then
          FindPairPos = P
          Exit Function
        End If
      End If
    Case TagFSeparator
      If Not vPair0 Then
        If vPair1 + vPair2 = 0 Then FindPairPos = P - 1
      End If
  End Select
Loop
End Function

Private Sub GetArguments(ByRef retArgs(), ByVal Text As String)
Dim N As Long, P0 As Long, P As Long, I As Long, C As String, fSkip As Boolean
Dim vPair0 As Boolean   ' quote (TagQuote)
Dim vPair1 As Long      ' [ ]   (TagOpen, TagClose)
Dim vPair2 As Long      ' ( )   (TagFOpen, TagFClose)
ReDim retArgs(0)
If Len(Text) = 0 Then Exit Sub
N = 1
I = 1
P0 = 1
Do
  Call FirstInStr(P, C, P0, Text, TagEscape, TagQuote, TagOpen, TagClose, TagFOpen, TagFClose, TagFSeparator)
  If P = 0 Then Exit Do
  P0 = P + Len(C)
  fSkip = False
  Select Case C
    Case TagEscape
      Select Case UCase$(Mid$(Text, P0, 1))
        Case "0", "T", "N"
          P0 = P0 + 1
        Case "*"
          If InStr(P0, Text, "*", vbBinaryCompare) = 0 Then
            P0 = P0 + 1
          Else
            P0 = InStr(P0, Text, "*", vbBinaryCompare) + 1
          End If
        Case Else
          P0 = P0 + 1
      End Select
    Case TagQuote
      vPair0 = Not vPair0
    Case TagOpen
      If Not vPair0 Then
        vPair1 = vPair1 + 1
      End If
    Case TagClose
      If Not vPair0 Then
        vPair1 = vPair1 - 1
      End If
    Case TagFOpen
      If Not vPair0 Then
        vPair2 = vPair2 + 1
      End If
    Case TagFClose
      If Not vPair0 Then
        vPair2 = vPair2 - 1
      End If
    Case TagFSeparator
      If Not vPair0 Then
        If vPair1 + vPair2 = 0 Then
          ReDim Preserve retArgs(0 To N)
          retArgs(N) = Mid$(Text, I, P - I)
          I = P0
          N = N + 1
        End If
      End If
  End Select
Loop
If I <= Len(Text) Then
  ReDim Preserve retArgs(0 To N)
  retArgs(N) = Mid$(Text, I)
End If
End Sub

Public Function Func(ByVal FuncName As String, ParamArray Arguments()) As String
Dim N As Long, I As Long
N = UBound(Arguments) - LBound(Arguments) + 1
I = LBound(Arguments) - 1
Func = NullValue
Select Case UCase$(FuncName)
  Case "ABBR"
    Select Case N
      Case 1
        Func = Func_ABBR(Arguments(I + 1))
      Case 2
        Func = Func_ABBR(Arguments(I + 1), Arguments(I + 2))
    End Select
  Case "LOWER"
    Select Case N
      Case 1
        Func = Func_LOWER(Arguments(I + 1))
    End Select
  Case "UPPER"
    Select Case N
      Case 1
        Func = Func_UPPER(Arguments(I + 1))
    End Select
  Case "PROPER"
    Select Case N
      Case 1
        Func = Func_PROPER(Arguments(I + 1))
    End Select
  Case "CAPS"
    Select Case N
      Case 1
        Func = Func_CAPS(Arguments(I + 1))
    End Select
  Case "NUM"
    Select Case N
      Case 1
        Func = Func_NUM(Arguments(I + 1), 0)
      Case 2
        Func = Func_NUM(Arguments(I + 1), Arguments(I + 2))
      Case 3
        Func = Func_NUM(Arguments(I + 1), Arguments(I + 2), Arguments(I + 3))
    End Select
  Case "FORMAT"
    Select Case N
      Case 1
        Func = Func_FORMAT(Arguments(I + 1))
      Case 2
        Func = Func_FORMAT(Arguments(I + 1), Arguments(I + 2))
    End Select
  Case "FMT"
    Select Case N
      Case 1
        Func = Func_FMT(Arguments(I + 1))
      Case 2
        Func = Func_FMT(Arguments(I + 1), Arguments(I + 2))
      Case 3
        Func = Func_FMT(Arguments(I + 1), Arguments(I + 2), Arguments(I + 3))
    End Select
  Case "IF"
    Select Case N
      Case 1
        Func = Func_IF(Arguments(I + 1), Null, Null)
      Case 2
        Func = Func_IF(Arguments(I + 1), Arguments(I + 2), Null)
      Case 3
        Func = Func_IF(Arguments(I + 1), Arguments(I + 2), Arguments(I + 3))
    End Select
  Case "NVL"
    Select Case N
      Case 1
        Func = Func_NVL(Arguments(I + 1), Null)
      Case 2
        Func = Func_NVL(Arguments(I + 1), Arguments(I + 2))
    End Select
  Case "PAD"
    Select Case N
      Case 2
        Func = Func_PAD(Arguments(I + 1), Arguments(I + 2))
    End Select
  Case "LEFT"
    Select Case N
      Case 2
        Func = Func_LEFT(Arguments(I + 1), Arguments(I + 2))
    End Select
  Case "RIGHT"
    Select Case N
      Case 2
        Func = Func_RIGHT(Arguments(I + 1), Arguments(I + 2))
    End Select
End Select
End Function

Public Function Func_ABBR(ByVal Text As String, Optional ByVal Length As Long) As String
If Text = NullValue Then Func_ABBR = NullValue: Exit Function
Dim I As Long, res As String
If Len(Text) <= Length Then
  Func_ABBR = Text
Else
  For I = 1 To Len(Text)
    If InStr(WordCharacters, Mid$(Text, I, 1)) > 0 Then
      If Mid$(Text, I, 1) = UCase$(Mid$(Text, I, 1)) Then
        res = res & Mid$(Text, I, 1)
      End If
    End If
  Next I
  Func_ABBR = res
End If
End Function

Public Function Func_LOWER(ByVal Text As String) As String
If Text = NullValue Then Func_LOWER = NullValue: Exit Function
Func_LOWER = LCase$(Text)
End Function

Public Function Func_UPPER(ByVal Text As String) As String
If Text = NullValue Then Func_UPPER = NullValue: Exit Function
Func_UPPER = UCase$(Text)
End Function

Public Function Func_PROPER(ByVal Text As String) As String
If Text = NullValue Then Func_PROPER = NullValue: Exit Function
Dim I As Long, res As String
res = UCase$(Mid$(Text, 1, 1))
For I = 2 To Len(Text)
  If InStr(1, WordSeparator, Mid$(Text, I - 1, 1), vbBinaryCompare) = 0 Then
    res = res & LCase$(Mid$(Text, I, 1))
  Else
    res = res & UCase$(Mid$(Text, I, 1))
  End If
Next I
Func_PROPER = res
End Function

Public Function Func_CAPS(ByVal Text As String) As String
If Text = NullValue Then Func_CAPS = NullValue: Exit Function
Dim I As Long, res As String
res = UCase$(Mid$(Text, 1, 1))
For I = 2 To Len(Text)
  If InStr(1, WordSeparator, Mid$(Text, I - 1, 1), vbBinaryCompare) = 0 Then
    res = res & (Mid$(Text, I, 1))
  Else
    res = res & UCase$(Mid$(Text, I, 1))
  End If
Next I
Func_CAPS = res
End Function

Public Function Func_NUM(ByVal Value, ByVal PadSize As Long, Optional ByVal PadChar As String = "0") As String
If IsNull(Value) Or IsEmpty(Value) Or (Value = NullValue) Then Func_NUM = NullValue: Exit Function
Func_NUM = Format$(Value, String$(PadSize, PadChar))
If Len(Func_NUM) > PadSize Then Func_NUM = String$(PadSize, "#")
End Function

Public Function Func_FORMAT(ByVal Value, Optional ByVal Mask) As String
If (Value = NullValue) Then Value = Null
Func_FORMAT = Format$(Value, Mask)
End Function

Public Function Func_FMT(ByVal Text, Optional ByVal Mask As String, Optional ByVal IgnoreChars As String) As String
Dim res As String, I As Long, C As Long, S As String, D As Long
If IsNull(Text) Or IsEmpty(Text) Or (Text = NullValue) Then Func_FMT = NullValue: Exit Function
If (Mask = NullValue) Then Func_FMT = NullValue: Exit Function
For I = 1 To Len(Mask)
  S = Mid$(Mask, I, 1)
  If S = "#" Then
    C = C + 1
    Do
      S = Mid$(Text, C + D, 1)
      If Len(S) > 0 Then Exit Do
      If Len(IgnoreChars) = 0 Then Exit Do
      If InStr(IgnoreChars, S) > 0 Then D = D + 1
    Loop
  End If
  res = res & S
Next I
Func_FMT = res
End Function

Public Function Func_IF(ByVal Field, ByVal IfTrue, ByVal IfFalse) As String
If IsNull(IfTrue) Or IsEmpty(IfTrue) Then IfTrue = NullValue
If IsNull(IfFalse) Or IsEmpty(IfFalse) Then IfFalse = NullValue
If IsEmpty(Field) Then Field = Null
If Not IsNull(Field) Then
  Select Case VarType(Field)
    Case VbVarType.vbBoolean
    Case VbVarType.vbByte
      Field = CBool(Field)
    Case VbVarType.vbInteger, VbVarType.vbLong
      Field = CBool(Field)
    Case VbVarType.vbSingle, VbVarType.vbDouble
      Field = CBool(Field)
    Case VbVarType.vbCurrency
      Field = CBool(Field)
    Case VbVarType.vbString
      Select Case UCase$(Trim$(Field))
        Case "1", "+", "Y", "YES", "TRUE", "ON"
          Field = True
        Case "0", "-", "N", "NO", "FALSE", "OFF"
          Field = False
        Case Else
          If IsNumeric(Field) Then Field = True Else Field = Null
      End Select
    Case VbVarType.vbVariant
      On Error Resume Next
      Field = CBool(Field)
      If Err > 0 Then Field = Null
      On Error GoTo 0
    Case VbVarType.vbDate
      Field = True
    Case VbVarType.vbEmpty
      Field = Null
    Case VbVarType.vbNull
      Field = Null
    Case Else
      Field = True
  End Select
End If
Func_IF = NullValue
If Not IsNull(Field) Then
  If Field Then
    If Not IsNull(IfTrue) Then Func_IF = IfTrue
  Else
    If Not IsNull(IfFalse) Then Func_IF = IfFalse
  End If
End If
End Function

Public Function Func_NVL(ByVal Field, ByVal IfNullField) As String
If IsNull(Field) Or IsEmpty(Field) Then Field = NullValue
If IsNull(IfNullField) Or IsEmpty(IfNullField) Then IfNullField = NullValue
If Field = NullValue Then
  Func_NVL = IfNullField
Else
  Func_NVL = Field
End If
End Function

Public Function Func_PAD(ByVal Text As String, ByVal PadSize As Long) As String
If Text = NullValue Then Func_PAD = NullValue: Exit Function
Func_PAD = Left$(Text & Space$(PadSize), PadSize)
End Function

Public Function Func_LEFT(ByVal Text As String, ByVal CharsCount As Long) As String
If Text = NullValue Then Func_LEFT = NullValue: Exit Function
Func_LEFT = Left$(Text, CharsCount)
End Function

Public Function Func_RIGHT(ByVal Text As String, ByVal CharsCount As Long) As String
If Text = NullValue Then Func_RIGHT = NullValue: Exit Function
Func_RIGHT = Right$(Text, CharsCount)
End Function

Public Function TrimAll(ByVal Text As String, Optional ByVal TrimChars As String = vbSpace) As String
Dim L As Long, I As Long
Do Until L = Len(Text)
  For I = 1 To Len(TrimChars)
    Text = Replace(Text, Mid$(TrimChars, I, 1), vbNullString)
  Next I
  L = Len(Text)
Loop
TrimAll = Text
End Function

Public Function FmtString(ByVal Text As String, ByVal FormatString As String, Optional ByVal IgnoreChars As String) As String
Dim C As Long, I As Long, D As Long, S As String
For I = 1 To Len(FormatString)
  S = Mid$(FormatString, I, 1)
  If S = "#" Then
    C = C + 1
    Do
      S = Mid$(Text, C + D, 1)
      If Len(S) = 0 Then Exit Do
      If Len(IgnoreChars) = 0 Then Exit Do
      If InStr(IgnoreChars, S) = 0 Then Exit Do
      D = D + 1
    Loop
    If Len(S) > 0 Then Mid$(FormatString, I, 1) = S
  End If
Next I
FmtString = FormatString
End Function

Public Function FmtText(ByVal FormatString As String, ParamArray Args() As Variant) As String
Dim res As String, I0 As Long, I As Long, N As Long, fTagOpen As Boolean, TagValue As Variant, TagFormat As String
I0 = 1
res = vbNullString
Do Until I0 > Len(FormatString)
  If fTagOpen Then
    Call FirstInStr(I, vbNullString, I0, FormatString, TagClose)
    If I = 0 Then I = Len(FormatString) + 1
    TagValue = Mid$(FormatString, I0 + Len(TagOpen), I - I0 - 1)
    fTagOpen = False
    I = I + 1
    N = InStr(TagValue, TagSeparator)
    If N > 0 Then
      TagFormat = Mid$(TagValue, N + Len(TagSeparator))
      TagValue = Left$(TagValue, N - 1)
    End If
    If Left$(TagValue, Len(TagField)) = TagField Then
      TagValue = Args(Val(Mid$(TagValue, Len(TagField) + 1)) - 1 + LBound(Args))
    Else
      TagValue = Null
    End If
    If Left$(TagFormat, Len(TagQuote)) = TagQuote And Right$(TagFormat, Len(TagQuote)) = TagQuote Then
      TagFormat = Mid$(TagFormat, Len(TagQuote) + 1, Len(TagFormat) - 2 * Len(TagQuote))
    Else
      TagFormat = vbNullString
    End If
    res = res & Format$(TagValue, TagFormat)
  Else
    Call FirstInStr(I, vbNullString, I0, FormatString, TagEscape, TagOpen)
    If I = 0 Then I = Len(FormatString) + 1
    res = res & Mid$(FormatString, I0, I - I0)
    If Mid$(FormatString, I, Len(TagEscape)) = TagEscape Then
      res = res & Mid$(FormatString, I + 1, 1)
      I = I + 2
    ElseIf Mid$(FormatString, I, Len(TagOpen)) = TagOpen Then
      TagFormat = vbNullString
      fTagOpen = True
    End If
  End If
  I0 = I
Loop
FmtText = res
End Function

Public Function FmtNum(ByVal Value As Long, Optional ByVal Pad As Long = 3, Optional ByVal Suffix As String = ".")
Dim V As String
V = Trim$(Str$(Value))
FmtNum = Space$(2 * (Pad - Len(V))) & V & Suffix
End Function

Public Function FmtName(ByVal Name As String) As String
Dim N() As String, I As Long, ret As String
N = Split(Name, NameHyphen)
For I = LBound(N) To UBound(N)
  ret = ret & UCase$(Left$(N(I), 1)) & LCase$(Mid$(N(I), 2))
  If I < UBound(N) Then ret = ret & NameHyphen
Next I
Erase N
FmtName = ret
End Function

Public Function FmtInitials(ByVal Name As String) As String
Dim N() As String, I As Long, ret As String
N = Split(Name, NameHyphen)
For I = LBound(N) To UBound(N)
  ret = ret & UCase$(Left$(N(I), 1)) & "."
  If I < UBound(N) Then ret = ret & NameHyphen
Next I
Erase N
FmtInitials = ret
End Function

Public Function FmtFullName(ByVal FullName As String) As String
Dim N() As String, N0 As String, N1 As String, N2 As String
N() = Split(FullName, NameSpacer)
If UBound(N) >= 0 Then N0 = N(0)
If UBound(N) >= 1 Then N1 = N(1)
If UBound(N) >= 2 Then N2 = N(2)
Erase N
FmtFullName = FmtName(N0) & NameSpacer & FmtName(N1) & NameSpacer & FmtName(N2)
End Function

Public Function FmtShortName(ByVal FullName As String) As String
Dim N() As String, N0 As String, N1 As String, N2 As String
N() = Split(FullName, NameSpacer)
If UBound(N) >= 0 Then N0 = N(0)
If UBound(N) >= 1 Then N1 = N(1)
If UBound(N) >= 2 Then N2 = N(2)
Erase N
FmtShortName = FmtName(N0) & NameSpacer & FmtInitials(N1) & NameSpacer & FmtInitials(N2)
End Function
